@@ -2,40 +2,72 @@
 Test suite containing functional unit tests for the exported primitives and
 classes.
 """
+from __future__ import annotations
+from typing import Tuple, Union, Optional, Callable, Iterable
+from unittest import TestCase
 from parts import parts
 from bitlist import bitlist
 from fountains import fountains
-from unittest import TestCase
 
 from fe25519.fe25519 import fe25519
 
-def one_from_bytes(bs: bytes):
+def one_from_bytes(bs: bytes) -> fe25519:
+    """
+    Generate one element from a given bit sequence obtained
+    using :obj:`fountains`.
+    """
     ps = list(parts(bs, length=8))
     return fe25519([int.from_bytes(p, 'little') for p in ps])
 
-def two_from_bytes(bs: bytes):
+def two_from_bytes(bs: bytes) -> Tuple[fe25519, fe25519]:
+    """
+    Generate two elements from a given bit sequence obtained
+    using :obj:`fountains`.
+    """
     ps = list(parts(bs, length=8))
     f1 = fe25519([int.from_bytes(p, 'little') for p in ps[:5]])
     f2 = fe25519([int.from_bytes(p, 'little') for p in ps[5:]])
     return (f1, f2)
 
-def check_or_generate(self, fs, bits):
-    if bits is not None:
-        self.assertTrue(all(fs)) # Check that all tests succeeded.
-    else:
+def check_or_generate(
+        testcase: TestCase,
+        fs: Union[Iterable[int], Iterable[bool]],
+        bits: Optional[str]
+    ) -> Optional[str]:
+    """
+    Wrapper that enables switching between performing a test or
+    generating specifications compatible with :obj:`fountains`.
+    """
+    if bits is None:
         return bitlist(list(fs)).hex() # Return target bits for this test.
 
-def check_or_generate_operation(self, fun, arity, bits):
+    testcase.assertTrue(all(fs)) # Check that all tests succeeded.
+    return None # Do not return a test input.
+
+def check_or_generate_operation(
+        testcase: TestCase,
+        fun: Union[Callable[[bytes], bytes], Callable[[bytes], bitlist]],
+        arity: int,
+        bits: Optional[str]
+    ) -> Optional[str]:
+    """
+    Wrapper that enables switching between performing a test or
+    generating specifications compatible with :obj:`fountains`.
+    """
     fs = fountains(
-        8*5*arity, 
+        8 * 5 * arity,
         seed=bytes(0), # This is also the default; explicit for clarity.
         limit=256,
         bits=bits,
         function=fun
     )
-    return check_or_generate(self, fs, bits)
+    return check_or_generate(testcase, fs, bits)
 
 class Test_fe25519(TestCase):
+    """
+    Tests for all class methods.
+    """
+    # pylint: disable=too-many-public-methods,missing-function-docstring
     def test_one(
             self,
             bits='b71ee55494c10540b2d3c4221793de6c6c722100387cab827ae1522affb5fd66'
@@ -104,7 +136,7 @@ class Test_fe25519(TestCase):
         ):
         fun = lambda bs: (one_from_bytes(bs).sq2()).to_bytes()
         return check_or_generate_operation(self, fun, 1, bits)
-  
+
     def test_pow22523(
             self,
             bits='4601cde640c8e05a4e63df3edc2a9d472851072b6b361eaaebcf781c0a116150'
@@ -148,7 +180,7 @@ class Test_fe25519(TestCase):
         ):
         def fun(bs):
             f0 = one_from_bytes(bs)
-            return bytes([f0 == f0])
+            return bytes([f0 == f0]) # pylint: disable=comparison-with-itself
         return check_or_generate_operation(self, fun, 1, bits)
 
     def test_eq_false(
@@ -196,7 +228,9 @@ class Test_fe25519(TestCase):
         ):
         def fun(bs):
             f = one_from_bytes(bs)
-            return bitlist([0 if eval(str(f)) == f else 255]).to_bytes()
+            return bitlist([
+                0 if eval(str(f)) == f else 255 # pylint: disable=eval-used
+            ]).to_bytes()
         return check_or_generate_operation(self, fun, 1, bits)
 
     def test_bytes(
@@ -217,8 +251,8 @@ class Test_fe25519(TestCase):
             return bitlist([0 if fe25519.from_bytes(bytes(f)) == f else 255]).to_bytes()
         return check_or_generate_operation(self, fun, 1, bits)
 
-if __name__ == "__main__":
-    # Generate reference bit lists for tests.
+if __name__ == '__main__':
+    # Generate specifications for tests.
     test_fe25519 = Test_fe25519()
     for m in [m for m in dir(test_fe25519) if m.startswith('test_')]:
         print(m + ': ' + getattr(test_fe25519, m)(bits=None))
